var CalendarTime = new function()
{
	// private fields
	var _imagePath = "common/images";
	var _dateFormat = "dd/MM/yyyy HH:mm";
	var _defaultDiv = "calDiv";
	var _defaultAnchor = "calAnchor";
	var _defaultTarget = "calTarget";
	var _underlay = null;
	var _validator = null;
	var _buttonLabel = "OK";
	var _timeLabel = "Time:";
	var _hoursLabel = "Hours";
	var _minutesLabel = "Minutes";
	var _monthNames = ["January","February","March","April","May","June","July","August","September","October","November","December"];
	var _dayNames = ["Sun","Mon","Tue","Wed","Thu","Fri","Sat"];
	var _showMonthAfterYear = false;
	var _weekStartDay = 0;

	this.addValidator = function(v)
	{
		_validator = v;
	}

	this.getUnderlay = function()
	{
		if(_underlay == null)
		{
			_underlay = document.createElement("iframe");
//			_underlay.src = "javascript:false";
			_underlay.frameBorder = 0;
			_underlay.style.position="absolute";
			_underlay.style.display="none";
			_underlay.calDiv = null;
			_underlay.resize = function(){ var c = this.calDiv; if(c == null) return; var s = this.style; s.top = c.style.top; s.left = c.style.left; s.height=c.offsetHeight; s.width=c.offsetWidth;}
			_underlay.attach = function(calDiv){this.calDiv = calDiv;this.style.zIndex = calDiv.style.zIndex - 1;	this.resize();	this.style.display = "block";}
			_underlay.detach = function(){this.calDiv = null;this.style.display="none";}
			document.body.appendChild(_underlay);
		}
		return _underlay;
	}

	this.setImagePath = function(path)
	{
		_imagePath = path;
	}

	this.setDateFormat = function(format)
	{
		_dateFormat = format.replace('.SSS','.000');
		// calendar.js depends on this
		window.CP_dateFormat = format.replace('.SSS','.000');

	}

	this.setMonthNames = function(monthNames /*Comma-Delimited String*/)
	{
		if(monthNames)
			_monthNames = monthNames.split(',');
	}

	this.setDayNames = function(dayNames /*Comma-Delimited String*/)
	{
		if(dayNames)
		{
			_dayNames = dayNames.split(',');
		}
	}
	this.setButtonLabel = function(label)
	{
		_buttonLabel = label;
	}
	this.setTimeLabel = function(label)
	{
		_timeLabel = label;
	}
	this.setHoursLabel = function(label)
	{
		_hoursLabel = label;
	}
	this.setMinutesLabel = function(label)
	{
		_minutesLabel = label;
	}
	this.setWeekStartDay = function(startDay)
	{
		_weekStartDay = startDay;
	}
	this.setShowMonthAfterYear = function(showMonthAfterYear)
	{
		_showMonthAfterYear = showMonthAfterYear;
	}

	// create the popup
	this.newPopup = function()
	{
		var dateFormat = DateUtil.getDefaultDateTimeFormat();
		var ctp;
		if(arguments.length == 3)
		{
			var name = arguments[0];
			var showTime = arguments[1];
			var showDropDowns=arguments[2];

			if(!showTime)
				dateFormat = DateUtil.getDefaultDateFormat();

			ctp = new CalendarTimePopup(_defaultDiv+name, _defaultAnchor+name, _defaultTarget+name , showTime, showDropDowns,_imagePath, dateFormat);
		}
		else if(arguments.length == 5)
		{
			// divName, anchorName, targetName, showTime, showDropDowns
			if(!arguments[3]) //showTime
				dateFormat = DateUtil.getDefaultDateFormat();
			ctp = new CalendarTimePopup(arguments[0],arguments[1],arguments[2],arguments[3],arguments[4],_imagePath, dateFormat);
		}
		else
		{
			alert("Invalid arguments to Calendar Popup");
			return null;
		}

		ctp.validator = _validator;
		ctp.buttonLabel = _buttonLabel;
		ctp.timeLabel = _timeLabel;
		ctp.hoursLabel = _hoursLabel;
		ctp.minutesLabel = _minutesLabel;
		ctp.setMonthNames.apply(ctp,_monthNames);
		ctp.setMonthAbbreviations.apply(ctp,_monthNames);
		ctp.setDayHeaders.apply(ctp,_dayNames);
		ctp.showMonthAfterYear = _showMonthAfterYear;
		ctp.weekStartDay = _weekStartDay;
		ctp.setSelectedDate = CTP_setSelectedDate;

		return ctp;
	}

}
// global convenience function for calendar links to locate the calendarTimeObject
function CTP_getCalendarObject(index)
{
	if(window.popupWindowObjects)
		return window.popupWindowObjects[index];

	alert("CalendarPopup not properly initialized");

	return null;
}
function CTP_setSelectedDate(selectedDate)
{
	this.initialSelectedDate = selectedDate;
}

function CalendarTimePopup(divName, anchor, targetInput, showTime, showDropDowns, imagePath, dateFormat)
{
	var cp = new CalendarPopup(divName);

	// add mouseup capturing to calendar div
	$('#'+divName).mousedown(CTP_captureCalendarClick);
	// properties
	cp.showTime = showTime;
	cp.anchor= anchor;
	cp.targetInput = targetInput;
	cp.imagePath = imagePath;
	cp.dateFormat = dateFormat;
	cp.isShowNavigationDropdowns=showDropDowns;
	cp.isShowYearNavigationInput=showDropDowns;
	cp.yearSelectStartOffset=110;

	// methods
	cp.getCalendar = CTP_getCalendarWithTime;
	cp.getTimePicker = CTP_getTimePicker;
	cp.selectDate = CTP_selectDate;
	cp.selectHour = CTP_selectHour;
	cp.selectMinute = CTP_selectMinute;
	cp.selectSecond = CTP_selectSecond;
	//allow externally set
	//cp.setReturnFunction(CTP_saveSelection);
	cp.saveSelection = CTP_saveSelection;
	cp.open = CTP_openCalendarTime;
	cp.writeMonthSelections = CTP_writeMonthSelections;
	cp.writeYearSelections = CTP_writeYearSelections;
	cp.writeMonthLabel = CTP_writeMonthLabel;
	cp.writeYearNavigation = CTP_writeYearNavigation;
	cp.writeMonthAndYearLabel = CTP_writeMonthAndYearLabel;
	cp.getTimeValues = CTP_getTimeValues;

	// override with custom popup/hide code
	cp.defaultShowPopup = cp.showPopup;
	cp.showPopup = CTP_showPopup;
	cp.defaultHidePopup = cp.hidePopup;
	cp.hidePopup = CTP_hidePopup;
	cp.defaultRefresh = cp.refresh;
	cp.refresh = CTP_refresh;

	cp.underlay = null;
	return cp;
}

function CTP_openCalendarTime()
{
	if (arguments.length == 1)
	{
		// only open from an "enter" keystroke event
		var e = $.event.fix(arguments[0]);
		if (e.which != 13)
			return;
		e.stopPropagation();
	}
	this.select(document.getElementById(this.targetInput),
			this.anchor,
			this.dateFormat);
}

function CTP_selectHour(x)
{
	this.currentDate.setHours(x);
}

function CTP_selectMinute(x)
{
	this.currentDate.setMinutes(x);
}

function CTP_selectSecond(x)
{
	this.currentDate.setSeconds(x);
}

function CTP_selectDate(y,m,d)
{
	var dt = this.currentDate;
	dt.setDate(1); //S0559679
    dt.setYear(y);
    dt.setMonth(m-1);
    dt.setDate(d);

	if(this.showTime)
	{
		this.selectHour(document.getElementById("cphour"+this.index).value);
		this.selectMinute(document.getElementById("cpminute"+this.index).value);
		this.showCalendar(this.anchor);
	}
	else
	{
		this.selectHour(0);
		this.selectMinute(0);
		this.selectSecond(0);
		this.saveSelection();
	}
}

function CTP_saveSelection()
{
	if(this.showTime)
	{
		this.selectHour(document.getElementById("cphour"+this.index).value);
		this.selectMinute(document.getElementById("cpminute"+this.index).value);
	}
	if(this.disabledDatesExpression.indexOf('>=')<1){//not a DOB

		if(this.validator != null)
			if(this.validator(this.currentDate) == false)
			{
				return;
			}
	}

	document.getElementById(this.targetInput).value = DateUtil.formatDate(this.currentDate,this.dateFormat);
	document.getElementById(this.targetInput).focus();

	if ( typeof this.returnFunction === "function" )
		this.returnFunction(this.currentDate);

	this.hideCalendar();
}

function CTP_refresh()
{
	this.defaultRefresh();
	if(this.underlay != null)
		this.underlay.resize();
}

function CTP_writeMonthSelections(month, year, refresh)
{
	var result ="";
	result += '<select name="cpMonth" onChange="'+refresh+'('+this.index+',this.options[this.selectedIndex].value-0,'+(year-0)+');">';
	for( var monthCounter=1;monthCounter<=12;monthCounter++){
		var selected =(monthCounter==month) ? 'SELECTED' : '';
		result += '<option value="'+monthCounter+'" '+selected+'>'+this.monthNames[monthCounter-1]+'</option>';
	}
	result += '</select>';
	return result;
}
function CTP_writeYearSelections(month, year, refresh)
{
	var now = new Date();
	var result ="";
	result += '<select name="cpYear" onChange="'+refresh+'('+this.index+','+month+',this.options[this.selectedIndex].value-0);">';
	for( var yearCounter=year-this.yearSelectStartOffset;yearCounter<=now.getFullYear();yearCounter++){
		var selected =(yearCounter==year) ? 'SELECTED' : '';
		result += '<option value="'+yearCounter+'" '+selected+'>'+yearCounter+'</option>';
	}
	result += '</select>';
	return result;
}
function CTP_writeMonthLabel(month, year, next_month, next_month_year, last_month, last_month_year, refreshLink)
{
	var result ="";
	result += '<TD CLASS="'+this.cssPrefix+'cpMonthNavigation" WIDTH="10"><A CLASS="'+this.cssPrefix+'cpMonthNavigation" HREF="'+refreshLink+'('+this.index+','+last_month+','+last_month_year+');">&lt;</A></TD>';
	result += '<TD CLASS="'+this.cssPrefix+'cpMonthNavigation" WIDTH="58"><SPAN CLASS="'+this.cssPrefix+'cpMonthNavigation">'+this.monthNames[month-1]+'</SPAN></TD>';
	result += '<TD CLASS="'+this.cssPrefix+'cpMonthNavigation" WIDTH="10"><A CLASS="'+this.cssPrefix+'cpMonthNavigation" HREF="'+refreshLink+'('+this.index+','+next_month+','+next_month_year+');">&gt;</A></TD>';
	result += '<TD CLASS="'+this.cssPrefix+'cpMonthNavigation" WIDTH="10">&nbsp;</TD>';
	return result;
}
function CTP_writeYearNavigation(month, year, refreshLink)
{
	var result = "";
	result += '<TD CLASS="'+this.cssPrefix+'cpYearNavigation" WIDTH="10"><A CLASS="'+this.cssPrefix+'cpYearNavigation" HREF="'+refreshLink+'('+this.index+','+month+','+(year-1)+');">&lt;</A></TD>';
	if(this.isShowYearNavigationInput){
		result += '<TD CLASS="'+this.cssPrefix+'cpYearNavigation" WIDTH="36"><INPUT NAME="cpYear" CLASS="'+this.cssPrefix+'cpYearNavigation" SIZE="4" MAXLENGTH="4" VALUE="'+year+'" onBlur="'+refresh+'('+this.index+','+month+',this.value-0);"></TD>';
	}
	else{
		result += '<TD CLASS="'+this.cssPrefix+'cpYearNavigation" WIDTH="36"><SPAN CLASS="'+this.cssPrefix+'cpYearNavigation">'+year+'</SPAN></TD>';
	}
	result += '<TD CLASS="'+this.cssPrefix+'cpYearNavigation" WIDTH="10"><A CLASS="'+this.cssPrefix+'cpYearNavigation" HREF="'+refreshLink+'('+this.index+','+month+','+(year+1)+');">&gt;</A></TD>';
	return result;
}

function CTP_handleMonthKeyDown(event, index, month, year)
{
 var e = $.event.fix(event);
  if (e.which == 27) {
      CP_hideCalendar(index);
      return;
  }
  if (e.which != 13)
      return;
  e.stopPropagation();
  CP_refreshCalendar(index, month,year);
}

function CTP_writeMonthAndYearLabel(month, year, next_month, next_month_year, last_month, last_month_year, refreshLink)
{
	var result ="";
	result += '<TD CLASS="'+this.cssPrefix+'cpMonthNavigation" WIDTH="15%" tabindex=0 onkeydown="CTP_handleMonthKeyDown(event,'+this.index+','+next_month+','+next_month_year+');" onclick="'+refreshLink+'('+this.index+','+last_month+','+last_month_year+');"><img src="'+this.imagePath+'/leftWhite.gif" border=0/></TD>\n';
	result += '<TD CLASS="'+this.cssPrefix+'cpMonthNavigation" WIDTH="70%"><SPAN CLASS="'+this.cssPrefix+'cpMonthNavigation">';
	result += this.showMonthAfterYear ? (year +' '+this.monthNames[month-1]) : (this.monthNames[month-1]+' '+year);
	result += '</SPAN></TD>\n';
	result += '<TD CLASS="'+this.cssPrefix+'cpMonthNavigation" WIDTH="15%" tabindex=0 onkeydown="CTP_handleMonthKeyDown(event,'+this.index+','+next_month+','+next_month_year+');" onclick="'+refreshLink+'('+this.index+','+next_month+','+next_month_year+');"><img src="'+this.imagePath+'/rightWhite.gif" border=0></TD>\n';

	return result;
}

function CTP_handleCalKeyDown(event, index, selected_year, selected_month, selected_date)
{
	var e = $.event.fix(event);
	switch(e.which)
	{
	case 27: // ESC
		CP_hideCalendar(index);
		return;
	case 13: // Return
		break;
	case 37: // left arrow
		var nextCell =$(document.activeElement).prev();
		if (nextCell)
		{
			nextCell.focus();
		}
		return;
	case 38: // arrow up
		var cell = $(document.activeElement);
		var cellIdx = cell.index();

		var prevRow = cell.parent().prev();
		if (prevRow && prevRow.index() > 0)
		{
			var nextCell = prevRow.children().eq(cellIdx);
			nextCell.focus();
		}
		return;
	case 39: // right arrow
		var nextCell =$(document.activeElement).next();
		if (nextCell)
		{
			nextCell.focus();
		}
		return;
	case 40: // arrow down
		var cell = $(document.activeElement);
		var cellIdx = cell.index();

		var nextRow = cell.parent().next();
		if (nextRow && nextRow.index() < 7)
		{
			var nextCell = nextRow.children().eq(cellIdx);
			nextCell.focus();
		}
		return;
	default:
		return;
	}
	e.stopPropagation();
	CTP_getCalendarObject(index).selectDate(selected_year,selected_month,selected_date);
}

function CTP_getCalendarWithTime(){

	var now = new Date();
	if(this.type == "WINDOW"){
		var windowref = "window.opener.";
	}
	else{
		var windowref = "";
	}
	var result = "";
	if(this.type == "WINDOW"){
		result += "<HTML><HEAD><TITLE>Calendar</TITLE>"+this.getStyles()+"</HEAD><BODY MARGINWIDTH=0 MARGINHEIGHT=0 TOPMARGIN=0 RIGHTMARGIN=0 LEFTMARGIN=0>\n";
		result += '<CENTER><TABLE WIDTH=100% BORDER=0 BORDERWIDTH=0 CELLSPACING=0 CELLPADDING=0>\n';
	}
	else{
		result += '<TABLE CLASS="'+this.cssPrefix+'cpContainer"  CELLSPACING=0 CELLPADDING=0>\n';
		result += '<TR><TD ALIGN=CENTER>\n';result += '<CENTER>\n';
	}
	if(this.displayType=="date" || this.displayType=="week-end"){
		if(this.currentDate==null){
			this.currentDate = this.initialSelectedDate ? this.initialSelectedDate : now;
		}
		if(arguments.length > 0){
			var month = arguments[0];
		}
		else{
			var month = this.currentDate.getMonth()+1;
		}
		if(arguments.length > 1 && arguments[1]>0 && arguments[1]-0==arguments[1]){
			var year = arguments[1];
		}
		else{
			var year = this.currentDate.getFullYear();
		}
		var daysinmonth= new Array(0,31,28,31,30,31,30,31,31,30,31,30,31);
		if( ((year%4 == 0)&&(year%100 != 0) ) ||(year%400 == 0) ){
			daysinmonth[2] = 29;
		}
		var current_month = new Date(year,month-1,1);
		var display_year = year;
		var display_month = month;
		var display_date = 1;
		var weekday= current_month.getDay();
		var offset = 0;
		offset =(weekday >= this.weekStartDay) ? weekday-this.weekStartDay : 7-this.weekStartDay+weekday ;
		if(offset > 0){
			display_month--;
			if(display_month < 1){
				display_month = 12;display_year--;
			}
			display_date = daysinmonth[display_month]-offset+1;
		}
		var next_month = month+1;
		var next_month_year = year;
		if(next_month > 12){
			next_month=1;
			next_month_year++;
		}
		var last_month = month-1;
		var last_month_year = year;
		if(last_month < 1){
			last_month=12;last_month_year--;
		}
		var date_class;
		if(this.type!="WINDOW"){
			result += "<TABLE WIDTH=100% BORDER=0 BORDERWIDTH=0 CELLSPACING=0 CELLPADDING=0>";
		}
		result += '<TR>\n';
		var refresh = windowref+'CP_refreshCalendar';
		var refreshLink = 'javascript:' + refresh;
		if(this.isShowNavigationDropdowns){
			result += '<td class="'+this.cssPrefix+'cpMonthNavigation">';
			result += (this.showMonthAfterYear) ? (this.writeYearSelections(month, year, refresh)+this.writeMonthSelections(month, year, refresh))
												: (this.writeMonthSelections(month, year, refresh)+this.writeYearSelections(month, year, refresh));
			result += '</td>';
		}
		else{
			if(this.isShowYearNavigation){
				result += (this.showMonthAfterYear) ? (this.writeYearNavigation(month, year, refreshLink)+this.writeMonthLabel(month, year, next_month, next_month_year, last_month, last_month_year, refreshLink))
													: (this.writeMonthLabel(month, year, next_month, next_month_year, last_month, last_month_year, refreshLink)+this.writeYearNavigation(month, year, refreshLink));
			}
			else{
				result += this.writeMonthAndYearLabel(month, year, next_month, next_month_year, last_month, last_month_year, refreshLink);
			}
		}
		result += '</TR></TABLE>\n';
		result += '<TABLE WIDTH=90% BORDER=0 CELLSPACING=0 CELLPADDING=1 ALIGN=CENTER>\n';
		result += '<TR>\n';
		for(var j=0;j<7;j++){
			result += '<TD CLASS="'+this.cssPrefix+'cpDayColumnHeader" WIDTH="14%"><SPAN CLASS="'+this.cssPrefix+'cpDayColumnHeader">'+this.dayHeaders[(this.weekStartDay+j)%7]+'</TD>\n';
		}
		result += '</TR>\n';
		for(var row=1;row<=6;row++){
			result += '<TR>\n';
			for(var col=1;col<=7;col++){
				var disabled=false;
				if(this.disabledDatesExpression!=""){
					var ds=""+display_year+LZ(display_month)+LZ(display_date);
					eval("disabled=("+this.disabledDatesExpression+")");
				}
				var dateClass = "";
				if((display_month == this.currentDate.getMonth()+1) &&(display_date==this.currentDate.getDate()) &&(display_year==this.currentDate.getFullYear())){
					dateClass = "cpCurrentDate";
				}
				else if(display_month == month){
					dateClass = "cpCurrentMonthDate";
				}
				else{
					dateClass = "cpOtherMonthDate";
				}
				if(disabled || this.disabledWeekDays[col-1]){
					result += '	<TD CLASS="'+this.cssPrefix+dateClass+'Disabled"><SPAN CLASS="'+this.cssPrefix+dateClass+'Disabled">'+display_date+'</SPAN></TD>\n';
				}
				else{
					var selected_date = display_date;
					var selected_month = display_month;
					var selected_year = display_year;
					if(this.displayType=="week-end"){
						var d = new Date(selected_year,selected_month-1,selected_date,0,0,0,0);
						d.setDate(d.getDate() +(7-col));
						selected_year = d.getYear();
						if(selected_year < 1000){
							selected_year += 1900;
						}
						selected_month = d.getMonth()+1;
						selected_date = d.getDate();
					}
					result += '	<TD CLASS="'+this.cssPrefix+dateClass+'" tabindex=0 onkeydown="CTP_handleCalKeyDown(event,'+this.index+','+selected_year+','+selected_month+','+selected_date+');" onclick="'+windowref+'CTP_getCalendarObject('+this.index+').selectDate('+selected_year+','+selected_month+','+selected_date+');">'+display_date+'</TD>\n';
				}
				display_date++;
				if(display_date > daysinmonth[display_month]){
					display_date=1;display_month++;
				}
				if(display_month > 12){
					display_month=1;display_year++;
				}
			}
			result += '</TR>';
		}
		var current_weekday = now.getDay() - this.weekStartDay;
		if(current_weekday < 0){
			current_weekday += 7;
		}
		result += '<TR>\n';
		result += '	<TD COLSPAN=7 ALIGN=CENTER CLASS="'+this.cssPrefix+'cpTodayText">\n';
		if(this.disabledDatesExpression!=""){
			var ds=""+now.getFullYear()+LZ(now.getMonth()+1)+LZ(now.getDate());
			eval("disabled=("+this.disabledDatesExpression+")");
		}
		if(disabled || this.disabledWeekDays[current_weekday+1]){
			result += '		<SPAN CLASS="'+this.cssPrefix+'cpTodayTextDisabled">'+this.todayText+'</SPAN>\n';
		}
		else{
			result += '';
		}
		result += '';
		result += '	</TD></TR></TABLE></CENTER>';
		result += this.getTimePicker(windowref);
		result+='</TD></TR></TABLE>\n';
		result += this.getTimeValues(windowref);
	}
	if(this.displayType=="month" || this.displayType=="quarter" || this.displayType=="year"){
		if(arguments.length > 0){
			var year = arguments[0];
		}
		else{
			if(this.displayType=="year"){
				var year = now.getFullYear()-this.yearSelectStartOffset;
			}
			else{
				var year = now.getFullYear();
			}
		}
		if(this.displayType!="year" && this.isShowYearNavigation){
			result += "<TABLE WIDTH=100% BORDER=0 BORDERWIDTH=0 CELLSPACING=0 CELLPADDING=0>";
			result += '<TR>\n';result += '	<TD CLASS="'+this.cssPrefix+'cpYearNavigation" WIDTH="22"><A CLASS="'+this.cssPrefix+'cpYearNavigation" HREF="javascript:'+windowref+'CP_refreshCalendar('+this.index+','+(year-1)+');"><img src="'+this.imagePath+'/leftWhite.gif" border=0></A></TD>\n';
			result += '	<TD CLASS="'+this.cssPrefix+'cpYearNavigation" WIDTH="100">'+year+'</TD>\n';
			result += '	<TD CLASS="'+this.cssPrefix+'cpYearNavigation" WIDTH="22"><A CLASS="'+this.cssPrefix+'cpYearNavigation" HREF="javascript:'+windowref+'CP_refreshCalendar('+this.index+','+(year+1)+');"><img src="'+this.imagePath+'/rightWhite.gif" border=0></A></TD>\n';
			result += '</TR></TABLE>\n';
		}
	}
	if(this.displayType=="month"){
		result += '<TABLE WIDTH=90% BORDER=0 CELLSPACING=1 CELLPADDING=0 ALIGN=CENTER>\n';
		for(var i=0;i<4;i++){
			result += '<TR>';
			for(var j=0;j<3;j++){
				var monthindex =((i*3)+j);
				result += '<TD WIDTH=33% ALIGN=CENTER><A CLASS="'+this.cssPrefix+'cpText" HREF="javascript:'+windowref+this.returnMonthFunction+'('+year+','+(monthindex+1)+');'+windowref+'CP_hideCalendar(\''+this.index+'\');" CLASS="'+date_class+'">'+this.monthAbbreviations[monthindex]+'</A></TD>';
			}
			result += '</TR>';
		}
		result += '</TABLE></CENTER></TD></TR></TABLE>\n';
	}
	if(this.displayType=="quarter"){
		result += '<BR><TABLE WIDTH=90% BORDER=1 CELLSPACING=0 CELLPADDING=0 ALIGN=CENTER>\n';
		for(var i=0;i<2;i++){
			result += '<TR>';
			for(var j=0;j<2;j++){
				var quarter =((i*2)+j+1);
				result += '<TD WIDTH=50% ALIGN=CENTER><BR><A CLASS="'+this.cssPrefix+'cpText" HREF="javascript:'+windowref+this.returnQuarterFunction+'('+year+','+quarter+');'+windowref+'CP_hideCalendar(\''+this.index+'\');" CLASS="'+date_class+'">Q'+quarter+'</A><BR><BR></TD>';
				}
			result += '</TR>';
		}
		result += '</TABLE></CENTER></TD></TR></TABLE>\n';
	}
	if(this.displayType=="year"){
		var yearColumnSize = 4;
		result += "<TABLE WIDTH=100% BORDER=0 BORDERWIDTH=0 CELLSPACING=0 CELLPADDING=0>";
		result += '<TR>\n';
		result += '	<TD CLASS="'+this.cssPrefix+'cpYearNavigation" WIDTH="50%"><A CLASS="'+this.cssPrefix+'cpYearNavigation" HREF="javascript:'+windowref+'CP_refreshCalendar('+this.index+','+(year-(yearColumnSize*2))+');"><img src="'+this.imagePath+'/leftWhite.gif" border=0></A></TD>\n';
		result += '	<TD CLASS="'+this.cssPrefix+'cpYearNavigation" WIDTH="50%"><A CLASS="'+this.cssPrefix+'cpYearNavigation" HREF="javascript:'+windowref+'CP_refreshCalendar('+this.index+','+(year+(yearColumnSize*2))+');"><img src="'+this.imagePath+'/rightWhite.gif" border=0></A></TD>\n';
		result += '</TR></TABLE>\n';
		result += '<TABLE WIDTH=90% BORDER=0 CELLSPACING=1 CELLPADDING=0 ALIGN=CENTER>\n';
		for(var i=0;i<yearColumnSize;i++){
			for(var j=0;j<2;j++){
				var currentyear = year+(j*yearColumnSize)+i;
				result += '<TD WIDTH=50% ALIGN=CENTER><A CLASS="'+this.cssPrefix+'cpText" HREF="javascript:'+windowref+this.returnYearFunction+'('+currentyear+');'+windowref+'CP_hideCalendar(\''+this.index+'\');" CLASS="'+date_class+'">'+currentyear+'</A></TD>';
			}
			result += '</TR>';
		}
		result+=this.getTimePicker(windowref);
		result += '</TABLE></CENTER></TD></TR></TABLE>';
		result += this.getTimeValues(windowref);
	}
	if(this.type == "WINDOW"){
		result += "</BODY></HTML>";
	}

	return result;
}

function CTP_getTimePicker(windowref){
	if (this.showTime){

		return ['<tr><td align="center"><div class="ctp-time-container">',
		'<div class="ctp-time-label">'+this.timeLabel+'</div>',
		'<div><input type="text" tabindex="1" onclick="'+windowref+'CTP_showTimeValues(event, this, \'#cphours'+this.index+'\')" value="'+LZ(this.currentDate.getHours())+'" size="1" maxlength="2" id="cphour'+this.index+'" name="cphour'+this.index+'"/></div>',
		'<div>:</div>',
		'<div><input type="text" tabindex="2" onclick="'+windowref+'CTP_showTimeValues(event, this, \'#cpminutes'+this.index+'\')" value="'+LZ(this.currentDate.getMinutes())+'" size="1" maxlength="2" id="cpminute'+this.index+'" name="cpminute'+this.index+'"/></div>',
		'<div class="ctp-time-ok-button"><input type="button" tabindex="3" onclick="'+windowref+'CTP_getCalendarObject('+this.index+').selectSecond(0);'+windowref+'CTP_getCalendarObject('+this.index+').saveSelection();" value="'+this.buttonLabel+'"/></div>',
		'</div></td></tr>'].join('');
	}
	return "";
}
function CTP_getTimeValues(windowref)
{
	return ['<div id="cphours'+this.index+'" class="ctp-time-values-container" onclick="'+windowref+'CTP_setTimeValue(event,this,\'#cphour'+this.index+'\')">',
	'<div class="ctp-time-values-caption">'+this.hoursLabel+'</div>',
	'<div class="ctp-time-values"><div>00</div><div>01</div><div>02</div><div>03</div><div>04</div><div>05</div><div>06</div><div>07</div></div>',
	'<div class="ctp-time-values"><div>08</div><div>09</div><div>10</div><div>11</div><div>12</div><div>13</div><div>14</div><div>15</div><div>16</div><div>17</div><div>18</div></div>',
	'<div class="ctp-time-values"><div>19</div><div>20</div><div>21</div><div>22</div><div>23</div></div>',
	'</div>',
	'<div id="cpminutes'+this.index+'" class="ctp-time-values-container" onclick="'+windowref+'CTP_setTimeValue(event,this,\'#cpminute'+this.index+'\')">',
	'<div class="ctp-time-values-caption">'+this.minutesLabel+'</div>',
	'<div class="ctp-time-values"><div>00</div><div>15</div><div>30</div><div>45</div></div>',
	'</div>'].join('');
}

function CTP_setTimeValue(event, valuesElem, inputElemId)
{
	var e = $.event.fix(event);
	if($(e.target).parent().is(".ctp-time-values")){
		e.stopPropagation();
		$(inputElemId).val($(e.target).text()).select(); // RAPTOR-2399
		$(valuesElem).hide();
	}
}

function CTP_showTimeValues(event, inputElem, valuesId)
{
	var e = $.event.fix(event);
	e.stopPropagation();
	inputElem.select();

	$(".ctp-time-values-container").hide();
	$(valuesId+" .ctp-time-values div").removeClass('ctp-time-value-selected');
	var theValue = $(valuesId+" .ctp-time-values").find("div:contains("+inputElem.value+")");
	if(theValue.length == 0) // RAPTOR-2399
	{
		theValue = $(valuesId+" .ctp-time-values").find("div:first");
	}
	else
	{
		theValue.addClass('ctp-time-value-selected');
	}

	$(valuesId).show();

	var inputElemPosition = $(inputElem).position();
	var valuePosition = theValue.position();

	var positionTop = inputElemPosition.top - valuePosition.top;
	var positionLeft = inputElemPosition.left - valuePosition.left;

	$(valuesId).css('top', positionTop-2);
	$(valuesId).css('left', positionLeft);
}
function CTP_showPopup(anchorname)
{
	this.defaultShowPopup(anchorname);

	if(this.divName!=null)
	{
		var calDiv = document.getElementById(this.divName);
		this.underlay = CalendarTime.getUnderlay();
		this.underlay.attach(calDiv);
	}
}
function CTP_hidePopup()
{
	this.defaultHidePopup();
	if(this.divName != null && this.underlay != null)
	{
		this.underlay.detach();
		this.underlay = null;
	}
}

//override calendar.js functions
function formatDate(date,format){ return DateUtil.formatDateTime(date,format); }
function getDateFromFormat(val,format) { return DateUtil.parseDateTime(val,format).getTime(); }
function parseDate(val){ return DateUtil.parseDate(val); }
function PopupWindow_hidePopupWindows() { for(var i=0;i<popupWindowObjects.length;i++){ popupWindowObjects[i].hidePopup();} }
// if document receives onmouseup, then calendar did not receive the click
function PopupWindow_attachListener(){ $(document).mousedown(function(){PopupWindow_hidePopupWindows();}); }
function CTP_captureCalendarClick(e) { e.stopPropagation(); }
function PopupWindow_hideIfNotClicked(e){}; // deprecated
function PopupWindow_isClicked(e){};// deprecated
