sap.ui.jsview("sas.fscf.views.Palette", {
    getControllerName: function () {
        return "sas.fscf.views.Palette";
    },
    
    _bExplore: false,
    
    isExplore: function() {
    	return this._bExplore;
    },
    
    setExplore: function(bExplore) {
    	this._bExplore = bExplore;
    },

    _aFilterFields: null,
    
    getFilterFields: function() {
    	return this._aFilterFields;
    },
    
    setFilterFields: function(aFilterFields) {
    	this._aFilterFields = aFilterFields;
    },
    
    padFilterFields: function(fields) {
    	if (this.isExplore()) {
    		// make sure SMH_MULTI_ORG_NODE_KEY, SMH_ACCT_TYPE & SMH_ACTIVITY_TYPE are always in filter fields, 
    		// as they allow the user to narrow down the population based on transaction scope and transaction type
    		var newArray = fields.slice(0); // clone the array
    		if ($.inArray("SMH_MULTI_ORG_NODE_KEY", newArray) === -1) {
    			newArray[newArray.length] = "SMH_MULTI_ORG_NODE_KEY";
    		} 		
    		if ($.inArray("SMH_ACCT_TYPE", newArray) === -1) {
    			newArray[newArray.length] = "SMH_ACCT_TYPE";
    		} 		
    		if ($.inArray("SMH_ACTIVITY_TYPE", newArray) === -1) {
    			newArray[newArray.length] = "SMH_ACTIVITY_TYPE";
    		} 		
        	return newArray;
    	}
    	return fields;
    },

    createContent: function(oController) {
    },

    lazyLoadContent: function() {
      if (this.getContent().length == 0) {
    	jQuery.sap.require("sas.hc.m.NavContainer");
    	jQuery.sap.require("sas.hc.m.Page");
        jQuery.sap.require("sap.m.Toolbar");
    	jQuery.sap.require("sap.ui.model.json.JSONModel");
        jQuery.sap.require("sas.hc.m.SearchField");
        jQuery.sap.require("sap.ui.model.Filter");
        jQuery.sap.require("sap.ui.model.FilterOperator");
        jQuery.sap.require("sas.hc.m.List");
        jQuery.sap.require("sap.m.ListMode");
        jQuery.sap.require("sas.hc.m.StandardListItem");
    	jQuery.sap.require("sas.hc.ui.unified.Menu");
        jQuery.sap.require("sas.hc.ui.unified.MenuItem");
    	jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-core');
    	jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-widget');
    	jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-mouse');
    	jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-sortable');
    	jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-droppable');
    	jQuery.sap.require('sap.ui.thirdparty.jqueryui.jquery-ui-draggable');

    	var _this = this;
    	
    	this.addStyleClass("cfPaletteView");

		var fieldtypesUrl = this.getFieldTypesUrl();
        this._model = new sap.ui.model.json.JSONModel(fieldtypesUrl);

        var oSearch = new sas.hc.m.SearchField({
        	id: this.sId + "Search",
            placeholder: sas.fscf.rb.getText("rules.rulesPage.guidedView.search.txt"),
            width: "100%"
        });
        oSearch.attachLiveChange(function(oEvent){
    		oGroupList.toggleStyleClass("invisible", oEvent.mParameters.newValue.length > 0);
    		oList.toggleStyleClass("invisible", !(oEvent.mParameters.newValue.length > 0));

            oList.getBinding("items").filter([
                 new sap.ui.model.Filter("altName",
                		                 sap.ui.model.FilterOperator.Contains,
                		                 oEvent.mParameters.newValue)
            ]);
        });

        var oGroupList = new sas.hc.m.List({
        	id: this.sId + "GroupList",
            growing: true,
            growingScrollToLoad: true,
            growingThreshold: 100,
            width: "100%",
            height: "100%"
        });

        var groupData = {
        		"groups" : [

        		    		{
        		    			"key" : "ACCOUNT",
        		    			"name" : sas.fscf.rb.getText("rules.rulesPage.guidedView.categories.ACCOUNT.txt")
        		    		},

        		    		{
        		    			"key" : "ACTIVITY",
        		    			"name" : sas.fscf.rb.getText("rules.rulesPage.guidedView.categories.ACTIVITY.txt")
        		    		},

        		    		{
        		    			"key" : "AUTHENTICATION",
        		    			"name" : sas.fscf.rb.getText("rules.rulesPage.guidedView.categories.AUTHENTICATION.txt")
        		    		},

        		    		{
        		    			"key" : "CHANNEL",
        		    			"name" : sas.fscf.rb.getText("rules.rulesPage.guidedView.categories.CHANNEL.txt")
        		    		},

        		    		{
        		    			"key" : "CUSTOMER",
        		    			"name" : sas.fscf.rb.getText("rules.rulesPage.guidedView.categories.CUSTOMER.txt")
        		    		},

        		    		{
        		    			"key" : "OTHER",
        		    			"name" : sas.fscf.rb.getText("rules.rulesPage.guidedView.categories.OTHER.txt")
        		    		},

        		    		{
        		    			"key" : "SYSTEM",
        		    			"name" : sas.fscf.rb.getText("rules.rulesPage.guidedView.categories.SYSTEM.txt")
        		    		}
        		 ]
        };

        oGroupList.setModel(new sap.ui.model.json.JSONModel(groupData));
        oGroupList.bindAggregation("items", {
        	path: "/groups",
            factory: function(sId, oContext) {
                var li = new sas.hc.m.StandardListItem(sId, {
                      title: oContext.getProperty("name"),
                      icon: 'sap-icon://hc/blank'
                });
                li._key = oContext.getProperty("key");
                return li;
            },
        	sorter: [
        	         new sap.ui.model.Sorter("name")
        	]
        });
        oGroupList.attachSelectionChange(function(oEvent) {
        	var groupKey = oEvent.mParameters.listItem._key;
        	var groupTitle = oEvent.mParameters.listItem.getTitle();
            oList2.getBinding("items").filter([
                 new sap.ui.model.Filter("category",
                   		                 sap.ui.model.FilterOperator.EQ,
                   		                 groupKey)
            ]);
            var page2Cont = document.getElementById(_this.sId + 'NavContainerPage2-cont');
            if (page2Cont) {
            	setTimeout(function () {
                    page2Cont.scrollTop = 0;
            	}, 300);
            }
            oPage2.setTitle(groupTitle);
            oNavCon.to(_this.sId + 'NavContainerPage2');
            oEvent.mParameters.listItem.setSelected(false);
        });
        if (this.isExplore()) {
            oGroupList.addStyleClass("invisible");        	
        }

        var oList = new sas.hc.m.List({
        	id: this.sId + "List",
            growing: true,
            growingScrollToLoad: true,
            growingThreshold: 100,
            width: "100%",
            height: "100%"
        });
        if (!this.isExplore()) {
        	oList.addStyleClass("invisible");
        }
        
        var fGrouper = function(oContext) {
        	var sType = oContext.getProperty("category") || "Undefined";
        	return { key: sas.fscf.rb.getText("rules.rulesPage.guidedView.categories." + sType + ".txt"), value: sas.fscf.rb.getText("rules.rulesPage.guidedView.categories." + sType + ".txt") }
        };

        oList.setModel(this._model);
        oList.bindAggregation("items", {
        	path: "/items",
            factory: function(sId, oContext) {
                var li = new sas.hc.m.StandardListItem(sId, {
                      title: oContext.getProperty("altName"),
                      icon: _this.getIconForVariableType(oContext.getProperty("variableType")),
                      tooltip: oContext.getProperty("description"),
                      counter: oContext.getProperty("count")
                });
                li._fieldId = oContext.getProperty("id");
                li._variableType = oContext.getProperty("variableType");
                li._min = oContext.getProperty("min");
                li._max = oContext.getProperty("max");
                return li;
            },
        	sorter: [
        	         new sap.ui.model.Sorter("category", false, fGrouper),
        	         new sap.ui.model.Sorter("count", true),
        	         new sap.ui.model.Sorter("altName")
        	]
        });

        if (this.getFilterFields() && this.getFilterFields().length > 0) {
        	var paddedFilterFields = this.padFilterFields(this.getFilterFields());
            oList.getBinding("items").filter([
                new sap.ui.model.Filter("altName", 
                	function(oValue) {
                		return ($.inArray(oValue, paddedFilterFields) !== -1);
                	})
                ]);
        }


        var makeListItemsDraggable = function(oEvent, oData) {
        	$("#" + oData + " li:not(.sapMListNoData, .sapMGHLI, .sapMCLI)").draggable({
        		helper: "clone",
        		appendTo: "body",
                scroll: false,
                cursorAt: {left: -5, top: -5},
                start: function(e, ui)
                {
                 $(ui.helper).addClass("cfPaletteViewDragging");
                }
        	});
        };
        oList.onAfterRendering = function() {
        	if (sas.hc.m.List.prototype.onAfterRendering) {
        		sas.hc.m.List.prototype.onAfterRendering.apply(this);
        	}
        	makeListItemsDraggable(null, oList.sId);
        };
        oList.attachUpdateFinished(oList.sId, makeListItemsDraggable);

        var oMenu = new sas.hc.ui.unified.Menu(this.sId + "_Menu");
        oList.setAggregation("contextMenu", oMenu);
        var customizeContextMenu = function(oMenu, oEvent) {
            var menuItems = oMenu.removeAllItems();
    		for (var i = menuItems.length - 1; i >=0; i--) {
    			menuItems[i].destroy();
    		}
            var oListItem = $(oEvent.target).control()[0];
            if (oListItem != null && oListItem.getMetadata().getName() == "sas.hc.m.StandardListItem") {
            	var addable = true;
                var oMenuItem = new sas.hc.ui.unified.MenuItem({
                	id: this.sId + "_AddMenuItem",
                    text: sas.fscf.rb.getText("rules.rulesPage.guidedView.palette.addToGrid.txt"),
                    enabled: addable
                });
                oMenuItem.attachSelect(function() {
                    _this.getGrid().addPanel(null, _this.getGrid().getContent()[0], "", oListItem.getTitle(), oListItem._fieldId, oListItem._variableType, "EQUAL", null, oListItem._min, oListItem._max);
                    var scrollPane = document.getElementById(_this.getGrid().getParent().sId + "-content-0");
                    setTimeout(function () {
                        scrollPane.scrollLeft = scrollPane.scrollWidth + 1000;  // scroll all the way to the right/end                    	
                    }, 5000);
                });
                oMenu.addItem(oMenuItem);
            }
            return oMenu;
        };
        oList.customizeContextMenu = customizeContextMenu;

        var oList2 = new sas.hc.m.List({
        	id: this.sId + "List2",
            growing: true,
            growingScrollToLoad: true,
            growingThreshold: 100,
            width: "100%",
            height: "100%"
        });

        oList2.setModel(this._model);
        oList2.bindAggregation("items", {
        	path: "/items",
            factory: function(sId, oContext) {
                var li = new sas.hc.m.StandardListItem(sId, {
                      title: oContext.getProperty("altName"),
                      icon: _this.getIconForVariableType(oContext.getProperty("variableType")),
                      tooltip: oContext.getProperty("description"),
                      counter: oContext.getProperty("count")
                });
                li._fieldId = oContext.getProperty("id");
                li._variableType = oContext.getProperty("variableType");
                li._min = oContext.getProperty("min");
                li._max = oContext.getProperty("max");
                return li;
            },
        	sorter: [
        	         new sap.ui.model.Sorter("count", true),
        	         new sap.ui.model.Sorter("altName")
        	]
        });

        oList2.onAfterRendering = function() {
        	if (sas.hc.m.List.prototype.onAfterRendering) {
        		sas.hc.m.List.prototype.onAfterRendering.apply(this);
        	}
        	makeListItemsDraggable(null, oList2.sId);
        };
        oList2.attachUpdateFinished(oList2.sId, makeListItemsDraggable);

        var oMenu2 = new sas.hc.ui.unified.Menu();
        oList2.setAggregation("contextMenu", oMenu2);
        oList2.customizeContextMenu = customizeContextMenu;

        var oPage = new sas.hc.m.Page({
	  	      id: this.sId + 'NavContainerPage1',
		      showHeader: false,
		      showFooter: false,
		      showSubHeader: !this.isExplore(),
		      subHeader: new sap.m.Toolbar({
		        	id: this.sId + "NavContainerPage1Toolbar",
		    	    content: [
		    	              oSearch
		    	    ]
		      }),
			  content: [
			            oGroupList,
			            oList
			  ]
		});

        var oPage2 = new sas.hc.m.Page({
	  	      id: this.sId + 'NavContainerPage2',
		      showNavButton: true,
		      showSubHeader: false,
		      showFooter: false,
			  content: [
			            oList2
			  ]
		});
        oPage2.attachNavButtonPress(function(oEvent) {
        	oNavCon.back();
        });

        var oNavCon = new sas.hc.m.NavContainer({
        	  id: this.sId + 'NavContainer',
        	  pages: [
        		oPage,
        		oPage2
        	  ]
       	});

        this.addContent(oNavCon);
      }
    },
    getFieldTypesUrl: function() {
		var fieldtypesUrl = "rest/";
		if (this.isExplore()) {
			fieldtypesUrl += "businessunits/fieldtypes?filter=dateTimeType=false";
		} else {
			fieldtypesUrl += "businessunits/" + sas.fscf.getCurrentPageView().jsonRule.businessUnitId + "/fieldtypes?counts=true&filter=ruleAccessible=true and dateTimeType=false";
			if (sas.fscf.getCurrentPageView().jsonRule.transactionTypeId &&
	       		sas.fscf.getCurrentPageView().jsonRule.transactionTypeId != "-1") {
	   			fieldtypesUrl += "&txnType=" + sas.fscf.getCurrentPageView().jsonRule.transactionTypeId;
			}
		}
		return fieldtypesUrl;
    },
    reload: function() {
		var _this = this;

		var fieldtypesUrl = this.getFieldTypesUrl();

		sas.ajax({url:fieldtypesUrl, type: "GET", headers:sas.fscf.getAjaxHeaders()})
	  	  .done(function(oData){
	          _this._model = new sap.ui.model.json.JSONModel(oData);
	      	 _this.reloadContinue();
		  }).fail(function(oObj,textStatus,errorThrown){
	  	      console.info("Failed to get fieldtypes.");
	          _this._model = new sap.ui.model.json.JSONModel({});
	      	 _this.reloadContinue();
		  });			
    },
    reloadContinue: function() {
        var oList = sap.ui.getCore().byId(this.sId + "List");
        if (oList) {
          //re-bind
          oList.getModel().setData(this._model.getData());
        }
        var oList2 = sap.ui.getCore().byId(this.sId + "List2");
        if (oList2) {
          //re-bind
          oList2.getModel().setData(this._model.getData());
        }    	
    },
    getGrid: function() {
    	return sap.ui.getCore().byId(this.sId.substr(0, this.sId.indexOf("PaletteView")) + "GridView");
    },
    getIconForVariableType: function(variableType) {
    	if (variableType == "Numeric") {
    		return "sap-icon://hc/numericType";
    	} else if (variableType == "Character") {
    		return "sap-icon://hc/characterType";
    	} else if (variableType == "Date") {
    		return "sap-icon://hc/date";
    	} else if (variableType == "Time") {
    		return "sap-icon://hc/time";
    	} else if (variableType == "DateTime") {
    		return "sap-icon://hc/dateTime";
    	} else if (variableType == "Currency") {
    		return "sap-icon://hc/currency";
    	}
    	return "";
    },
    enableListItem: function(fieldType, bEnable) {
    	var oList = sap.ui.getCore().byId(this.sId + "List");
    	var oListItems = oList.getItems();
    	for (var i = 0; i < oListItems.length; i++) {
    		if (oListItems[i].getTitle() == fieldType) {
    			if (bEnable) {
        			$("#" + oListItems[i].sId).draggable("enable"); // turn on drag
    			} else {
        			$("#" + oListItems[i].sId).draggable("disable"); // turn off drag
    			}
    			break;
    		}
    	}
    	oList = sap.ui.getCore().byId(this.sId + "List2");
    	oListItems = oList.getItems();
    	for (var i = 0; i < oListItems.length; i++) {
    		if (oListItems[i].getTitle() == fieldType) {
    			if (bEnable) {
        			$("#" + oListItems[i].sId).draggable("enable"); // turn on drag
    			} else {
        			$("#" + oListItems[i].sId).draggable("disable"); // turn off drag
    			}
    			break;
    		}
    	}
    },
    // TODO: move this min and max info from the palette rest call to the panel rest call, as this is inefficient AND
    // there is a timing problem where the palette may not be done loading when this is asked for
    getFieldInfoForFieldId: function(fieldId) {
    	if (this._model.getData().items) {
        	for (var i = 0; i < this._model.getData().items.length; i++) {
        		if (this._model.getData().items[i].id == fieldId) {
        			return this._model.getData().items[i];
        		}
        	}
    	} else {
    		//console.info("failed to get field info for field id " + fieldId + " because palette is not done loading");
    	}
    	return null;
    },
    containsField: function(fieldName) {
    	if (this._model.getData().items) {
        	for (var i = 0; i < this._model.getData().items.length; i++) {
        		if (this._model.getData().items[i].altName == fieldName) {
        			return true;
        		}
        	}
    	}
    	return false;
    }
});
