jQuery.sap.declare("sas.fscf.views.PrimaryView");

sas.fscf.views.PrimaryView = {

    COLUMN_TYPE_TEXT: 0,
    COLUMN_TYPE_ICON: 1,
    COLUMN_TYPE_RATIO_HISTORY: 2,
		
    createContent: function(oController) {
    	jQuery.sap.require("sas.hc.ui.commons.layout.BorderLayout");
    	jQuery.sap.require("sas.hc.ui.commons.layout.BorderLayoutArea");
        jQuery.sap.require("sas.hc.m.ResponsiveToolbar");
        jQuery.sap.require("sas.hc.m.ToolbarSpacer");
    	jQuery.sap.require("sas.hc.m.Select");
        jQuery.sap.require("sas.hc.m.Label");
        jQuery.sap.require("sas.hc.m.Button");
        jQuery.sap.require("sas.hc.m.SegmentedButton");
        jQuery.sap.require("sas.hc.m.MenuButton");
        jQuery.sap.require("sas.hc.m.Menu");
        jQuery.sap.require("sas.hc.m.MenuItem");
        jQuery.sap.require("sas.hc.ui.layout.Splitter");
        jQuery.sap.require("sas.hc.ui.layout.SplitterLayoutData");
		jQuery.sap.require("sas.hc.ui.table.Table");
		jQuery.sap.require("sas.hc.ui.table.Column");
		jQuery.sap.require("sas.hc.m.Text");
		jQuery.sap.require("sas.fscf.control.Icon");
		jQuery.sap.require("sas.fscf.control.RatioHistory");

        var _this = this;
        
    	this.addStyleClass("cfPrimaryView");

        var oToolbar = new sas.hc.m.ResponsiveToolbar({
        	id: this.sId + "Toolbar",
    	    content: [
    	    ]
    	});

        var oTable = this.getTable();
        
        var oSplitter1LayoutData2 = new sas.hc.ui.layout.SplitterLayoutData({
        	resizable: true,
        	size: "auto"
        });
        
        var oSplitter1 = new sas.hc.ui.layout.Splitter({
            id: this.sId + 'Splitter1',
            contentAreas: [oTable],
            orientation: sap.ui.core.Orientation.Vertical,
            width: "100%",
            height: "100%"
        });
	        
        var oBorderLayout = new sas.hc.ui.commons.layout.BorderLayout({
            id: this.sId + "BorderLayout",
    		top: new sas.hc.ui.commons.layout.BorderLayoutArea({
                id: this.sId + "BorderLayoutTop",
    			size: "2rem",
    			content: [
    			          oToolbar
    			]
    		}),
    	    center: new sas.hc.ui.commons.layout.BorderLayoutArea({
                id: this.sId + "BorderLayoutCenter",
    			content: [
    			          oSplitter1
      			]
      		})
    	});
	        

    	setTimeout(function() {
        	_this.update(true, _this.getSearchTerm());
    	}, 500);
	    	
        return oBorderLayout;
    },
    
	getColumns: function() {
		return this._columns[this.getTableKey()];
	},
	
    getTable: function() {
    	var key = this.getTableKey();
    	if (!(typeof key === 'string' || key instanceof String)) {
    		console.info("table key must be a string");
    		return;
    	}
    	if (key.length < 1) {
    		console.info("table key must be a non-blank string");
    		return;
    	}
    	if (!this._tables) {
    		this._tables = {};
    	}
    	if (this._tables[key]) {
    		return this._tables[key];
    	}
    	var _this = this;
		var model = new sap.ui.model.json.JSONModel({});

        var oSplitter1LayoutData1 = new sas.hc.ui.layout.SplitterLayoutData({
        	resizable: true,
            size: (this._bShowPanels) ? "50%" : "100%"
        });
    	
		var oTable = new sas.hc.ui.table.Table({
			id: this.sId + 'Table' + key,
			editable: false,
			showTableOptions: false,
			enablePixelScrolling: false,
			visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
			selectionMode: sas.hc.ui.table.SelectionMode.MultiToggle,
			selectionBehavior: sas.hc.ui.table.SelectionBehavior.Row,
            showSelectionControls: true,
            enableDoubleClick: true,
            showAlternateRowShading: true,
            layoutData: oSplitter1LayoutData1
		});

		oTable.setModel(model);
		oTable.bindRows("/items");
			
		oTable.attachRowSelectionChange(function(oEvent) {
			var numRowsSelected = oEvent.oSource.getSelectedIndices().length;
			sas.fscf.removeAllMessageStrips();
			_this.updateToolbarEnablement(_this.getTableSelectedRowsData());
			_this.updatePanels();
		});
			
		oTable.attachDoubleClick(function(oEvent) {
			oEvent.oSource.setSelectedIndex(oEvent.mParameters.rowIndex);
			_this.doDefaultAction();
        });
		
		this._tables[key] = oTable;
		
		return oTable;
    },

	update: function(bUpdateColumns) {
   		this.updateToolbar();
   		this.updateTable(bUpdateColumns, this.getSearchTerm());
	},
	
	updateTable: function(bUpdateColumns, searchTerm) {
		var _this = this;

		if (!searchTerm) {
			searchTerm = this.getSearchTerm();
		}
		
		var splitter = sap.ui.getCore().byId(_this.sId + 'Splitter1'); 
		var table = splitter.getContentAreas()[0];
	  	  
		// clear any selection
		table.setSelectedIndex(-1);

		/* It is too slow to remove and add columns from a table or even to hide and show columns in a table,
		 * so instead for performance we support multiple tables and swap one table in for another
		 */
		if (bUpdateColumns) {
			var newTable = this.getTable();
			if (newTable != table) {
				splitter.removeContentArea(0);
				table = newTable;
				splitter.insertContentArea(table, 0);
			}
		}
		table.setBusy(true);
		
		// add the columns if they have not already been added
		if (table.getColumns().length == 0) {
			var columns = this.getColumns();
			for (var i = 0; i < columns.length; i++) {
				if (!columns[i].exclude) {
					table.addColumn(
						new sas.hc.ui.table.Column({
							label: _this.getHeaderLabel(columns[i].label),
							template: _this.getTemplate(columns[i]),
							sortProperty: columns[i].property,
							visible: true,
							width: columns[i].width + "px",
							hAlign: (columns[i].hAlign) ? columns[i].hAlign : sap.ui.core.HorizontalAlign.Begin 
						})
					);				
				}
			}
			table.sort(table.getColumns()[0], sas.hc.ui.table.SortOrder.Descending);
		}
		
		if (bUpdateColumns) {
			if (_this.afterUpdateColumns) {
				_this.afterUpdateColumns();
			}
		}

        sas.ajax({url:_this.getTableUrl(searchTerm), contentType:"application/json; charset=utf-8", dataType:"json", headers:sas.fscf.getAjaxHeaders()})
          .done(function(oData){
      		  // update the rows and re-bind the data model
        	  table.getModel().setData(oData);
  			  table.setBusy(false);
  			  _this.updateToolbarEnablement(_this.getTableSelectedRowsData());
        }).fail(function(oObj,textStatus){
        	console.info("an error occurred getting the table items");
			table.setBusy(false);
        });		
	},
	
	getTableSelectedRowsData: function() {
		var _this = this;

		var table = _this.getTable();

		var rowsData = [];
		var rowData;
		for (var i = 0; i < table.getSelectedIndices().length; i++) {
			// table could be sorted, so factor in index from binding
			rowData = table.getModel().getData().items[table.getBinding("rows").aIndices[table.getSelectedIndices()[i]]];
			if (rowData) {
				rowsData[rowsData.length] = rowData;
			}
		}
		
		return rowsData;
	},
	
	getTableRowsData: function() {
		var _this = this;

		var table = _this.getTable();

		return table.getModel().getData().items;		
	},
	
	getTableHasAnyRows: function() {
		var _this = this;
		
		var bAnyRows = false;
		try {
			bAnyRows = _this.getTable().getModel().getData().items.length > 0;
		} catch (e) {			
		}
		
		return bAnyRows;
	},
	
	setTableBusy: function(bBusy) {
		this.getTable().setBusy(bBusy);
	},
	
	getHeaderLabel: function(label) {
		var oLabel = new sas.hc.m.Label({
			text: label
		});
    	return oLabel;
	},
	
	getTemplate: function(column) {
		if (column.type == this.COLUMN_TYPE_ICON) {
			return new sas.fscf.control.Icon({
				src: {
					path: column.property,
					parts: column.parts,
					formatter: this.getIconURI(column.formatter)
                },
                tooltip: {
                	path: column.property,
					parts: column.parts,
                	formatter: this.getIconTooltip(column.formatter)
                },
                opacity: {
                	path: column.property,
					parts: column.parts,
                	formatter: this.getIconOpacity(column.formatter)
                }
			});
		} else if (column.type == this.COLUMN_TYPE_RATIO_HISTORY) {
			return new sas.fscf.control.RatioHistory({
				term: {
					path: column.parts[0]
                },
                term2: {
					path: column.parts[1]
                },
				termDescription: column.partsDescriptions[0],
                term2Description: column.partsDescriptions[1],
                history: {
                	path: column.parts[2]                
                }
			});
		} else {
			return new sas.hc.m.Text({
				text: {
					path: (column.parts) ? undefined : column.property,
					parts: (column.parts) ? column.parts : undefined,
                    formatter: this.getTextFormatter(column.formatter)
                },
                tooltip: column.tooltip ? {
                	path: column.tooltip,
                	formatter: function(val) {if (val == null || val == undefined) {return undefined;} else {return "" + val;}}
                } : undefined,
				wrapping: false
			});			
		}
	},
	
	performSearch: function(term) {
		this.updateTable(false, term);
	},	

    getSearchTerm: function() {
    	var oPageView = sas.fscf.getCurrentPageView();
    	if (oPageView && oPageView.getSearchTerm) {
    		return oPageView.getSearchTerm();
    	}
    	return undefined;
    }

    /*******************************************************************
     * The following items must be implemented in the
     * view class that mixes in PrimaryView
     * 
     * For performance (it is too slow to remove and add columns 
     * from a table or even to hide and show columns in a table), 
     * PrimaryView supports multiple tables and swaps one table in for 
     * another. So, if you need different columns in your table at 
     * different times, define multiple keys in your _columns in 
     * subclass such as {"Key1":[], "Key2":[]}. If you only have one 
     * set of columns that never change, then define just one key in
     * your _columns in subclass such as {"Main":[]}. Then implement 
     * getTableKey to return which table (key) you need at which times. 
     *******************************************************************
	_columns: {},

	getTableKey: function() {
	},
	
	//optional
	afterUpdateColumns: function() {
	},
	
    _bShowPanels: false,
    
	updateToolbar: function() {
	},
	
	updateToolbarEnablement: function(selectedRowsData) {
	},
	
	updatePanels: function() {
	},
	
    getTableUrl: function(searchTerm) {
	},

	doDefaultAction: function() {
	},
	
	getTextFormatter: function(type) {
	},
	
	//optional - only if using columns of type COLUMN_TYPE_ICON
	getIconURI: function(type) {
	},
	
	//optional - only if using columns of type COLUMN_TYPE_ICON
	getIconTooltip: function(type) {
	},
	
	//optional - only if using columns of type COLUMN_TYPE_ICON
	getIconOpacity: function(type) {
	}
     ******************************************************************/
};
