jQuery.sap.require("sas.fscf.views.estimation.EstimationHelper");
jQuery.sap.require("sas.fscf.views.estimation.EstimationResultsHelper");

sap.ui.jsfragment("sas.fscf.views.estimation.LogDialog", {

	createContent: function (controller) {/* jshint -W034 */
		"use strict";                     /* jshint +W034 */

        const EstimationHelper = sas.fscf.views.estimation.EstimationHelper;
        const SpacerFactory = EstimationHelper.SpacerFactory;
        const getText = EstimationHelper.getText;
        const injectHtmlIntoIframe = EstimationHelper.injectHtmlIntoIframe;
        const waitThenDo = EstimationHelper.waitThenDo;
        const logFrameId = "logFrame";
        const sId = this.sId;
        let requestorView = null;

        const logSelect = new sas.hc.m.Select({
            id: sId + "LogSelect",
            items: [],
        } );

        const logSelectHBox = new sas.hc.m.HBox({
            justifyContent: sap.m.FlexJustifyContent.Start,
            items: [
                new sas.hc.m.Text({text: getText("rules.estimation.log.input.label")}),
                SpacerFactory.horizontalSpacer("1rem"),
                logSelect,
            ],
        } );

        const contentVSpacer = SpacerFactory.verticalSpacer("1rem");
        const logSelectElements = [logSelectHBox, contentVSpacer];

        const get$LogFrame = function () {
            return $("#" + logFrameId);
        };

        const logVBox = new sas.hc.m.VBox({
            id: sId + "LogVBox",
            justifyContent: sap.m.FlexJustifyContent.Start,
            items: logSelectElements,
            height: "100%",
        } );

        const initLogFrame = function () {
            const $logVBox = logVBox.$();

            if (!$logVBox.find("iframe")[0]) {
                $logVBox.append("<iframe id='" + logFrameId + "' src='' style='width:100%; height:99%;' frameborder='0' marginwidth='0' marginheight='0'></iframe>");
                sas.fscf.sessionBindIFrames();
            }
        };

        const setUpLogFrame = function (dialog) {
            initLogFrame();

            const $logFrame = get$LogFrame();                                                             /* jshint -W117 */
            const hostAndPath = window.location.href.substring(0, window.location.href.lastIndexOf("/")); /* jshint +W117 */

            // Head tag
            const jQueryScript = "<script src='" + hostAndPath + "/common/scripts/jquery.js'></script>";  // RAPTOR-2399
            const sasLogScript = "<script src='" + hostAndPath + "/resources/sas/fscf/views/estimation/sas-log.js'></script>";
            const headTag = "<head>" + jQueryScript + sasLogScript + "</head>";

            // Body tag
            const log = dialog.getModel().getData().log;
            const bodyTag = "<body style='font-size: 12px;'>" + log.logRendered + "</body>";

            injectHtmlIntoIframe($logFrame[0], (headTag + bodyTag));
        };

        const showLog = function (dialog, logId) {
            sas.fscf.removeAllMessageStrips();
            requestorView.setBusy(true);

            dialog.close();

            const condition =  function () {
                return !dialog.isOpen();
            };
            const callback = function () {
                controller.showLog(requestorView, logId);
            };
            waitThenDo(condition, callback);
        };

        const setUpLogSelectElements = function (dialog) {
            const log = dialog.getModel().getData().log;
            const logCount = log.estimation.threadCount;

            if (logCount === 0) {
                logSelectElements.forEach(function (each) {
                    logVBox.removeItem(each);
                } );
                return;
            }

            for (let i = 1; i <= logCount; i++) {
                const value = i.toString();
                const item = new sas.hc.ui.core.Item({
                    key: value,
                    text: value,
                } );
                logSelect.addItem(item);
            }
            if (log.logNumber) {
                logSelect.setSelectedKey(log.logNumber);
            }
            logSelect.attachChange(function (event) {
                const selectedLogId = event.getSource().getSelectedKey();
                showLog(dialog, selectedLogId);
            } );
        };

        const logDialog = new sas.hc.m.Dialog({
            id: sId,
            title: {
                path: "/log",
                formatter: function (log) {
                    const estimation = log.estimation;
                    return getText("rules.estimation.log.fmt.title", [estimation.name, estimation.id]);
                }
            },
    	    content: [logVBox],
			contentHeight: "95%",
        	contentWidth: "95%",
    	    endButton: EstimationHelper.newCloseButton(),
            beforeOpen: [controller.handleLogDialogBeforeOpen, controller],
            afterClose: function (event) {
                logSelectHBox.destroy();
                event.getSource().destroy();

                controller.handleLogDialogAfterClose();
            },
		    } ).addEventDelegate({
            onBeforeRendering: function (event) {
                const dialog = event.srcControl;
                setUpLogSelectElements(dialog);
            },
            onAfterRendering: function (event) {
                const dialog = event.srcControl;
                setUpLogFrame(dialog);
            },
        } );

        logDialog.setRequestorView = function (view) {
            requestorView = view;
            return this;
        };

        logDialog.showErrorMessage = function (message) {
            sas.fscf.showErrorMessageStrip(message, sId);
            return this;
        };

        return logDialog;
    },
} );
