jQuery.sap.require("sas.fscf.views.estimation.HcHelper");
jQuery.sap.require("sas.fscf.views.estimation.EstimationHelper");
jQuery.sap.require("sas.fscf.views.estimation.EstimationResultsHelper");

sap.ui.jsfragment("sas.fscf.views.estimation.MatchedTransactionsDialog", {

	createContent: function (controller) {/* jshint -W034 */
		"use strict";                     /* jshint +W034 */

        const HcHelper = sas.fscf.views.estimation.HcHelper;
        const reduceSpaceAboveDialogButtons = HcHelper.reduceSpaceAboveDialogButtons;
        const EstimationHelper = sas.fscf.views.estimation.EstimationHelper;
        const getText = EstimationHelper.getText;
        const waitThenDo = EstimationHelper.waitThenDo;

        const Icon = sas.fscf.views.estimation.EstimationResultsHelper.Icon;

        const ONE_HUNDRED_PERCENT = "100%";
        const myId = this.sId;
        const alertsTabId = myId + "AlertsTab";
        const dialogId = myId + "MatchedTransactionsDialog";
        const tabBarId = myId + "TabBar";
        let estimationRuleMetrics = null;

        const alertsView =sap.ui.view({
            type: sap.ui.core.mvc.ViewType.JS,
            id: myId + "AlertRowsView",
            viewName: "sas.fscf.views.AlertRows",
            height: "100%",
        } );

        const alertsTab = new sas.hc.m.IconTabFilter({
            id: alertsTabId,
            text: getText("rules.estimation.alerts.label"),
            design: "Horizontal",
            content: [alertsView],
        } );

        const matchedTransactionsView = sap.ui.view({
            type: sap.ui.core.mvc.ViewType.JS,
            id: myId + "MatchedTransactionsView",
            viewName: "sas.fscf.views.RelatedRows",
            height: "100%",
        } );

        const matchedTransactionsTab = new sas.hc.m.IconTabFilter({
            id:  myId + "MatchedTransactionsTab",
            text: getText("rules.estimation.transaction.decisions.label"),
            design: "Horizontal",
            content: [matchedTransactionsView],
        } );

        const setHeightTo100Percent = function ($elt) {
        	$elt.css("height", "100%");
        };

        const doSetToolTip = function (tab, table, isAlertsView) {
            const itemCount = table.getModel().getData().items.length;
            const expectedTotal = isAlertsView
                ? estimationRuleMetrics.ruleDecisionMetrics.entityDecisionCounts.alertEntitiesCount
                : estimationRuleMetrics.forAlerts
                    ? -1     // In the case of alerted transactions, the back end code needs to be modified to provide the count.
                    : estimationRuleMetrics.ruleDecisionMetrics.transactionDecisionCounts.predictedConditionPositive;
            const tooltip = (expectedTotal === -1)
                ? (itemCount).toString()
                : getText("fcm.application.of.fmt.txt", [itemCount, expectedTotal]);

                tab.setTooltip(tooltip);
        };

        const setTabTooltip = function (view) {
            const table = view.getTable();
            if (table.getModel().getData().items) {
                const isAlertsView = (view === alertsView);
                const tab = isAlertsView ? alertsTab : matchedTransactionsTab;

                if (!tab.getTooltip()) {
                    doSetToolTip(tab, table, isAlertsView);
                }
            }
        };

        const setUpTable = function (view) {
            if (view && view.getTable()) {
                const tableId = view.getTable().sId + "-table";
                const $table = $("#"+tableId);

                setHeightTo100Percent($table);                   // "#RulesPageEstimationsViewMatchedTransactionsDialogMatchedTransactionsView_TransactionTable-table"
                setHeightTo100Percent($table.parent());          // "#RulesPageEstimationsViewMatchedTransactionsDialogMatchedTransactionsView_TransactionTable-tableCtrlCnt"
                setHeightTo100Percent($table.parent().parent()); // "#RulesPageEstimationsViewMatchedTransactionsDialogMatchedTransactionsView_TransactionTable-sapUiTableCtrlScr"
            }
        };

        const resizeTabContainerContent = function () {
            const dialogScrollContentId = dialogId + "-scrollCont";
            const $dialogScrollContent = $("#"+dialogScrollContentId);

            const tabBarHeaderId = tabBarId + "--header";              // RulesPageEstimationsViewMatchedTransactionsDialogTabBar--header
            const $tabBarHeader = $("#"+tabBarHeaderId);

            const containerContentId = tabBarId + "-containerContent"; // RulesPageEstimationsViewMatchedTransactionsDialogTabBar-containerContent
            const containerContentHeight = $dialogScrollContent.height() - $tabBarHeader.height() - 35;
            $("#"+containerContentId).height(containerContentHeight);

            const contentId = tabBarId + "-content";                   // RulesPageEstimationsViewMatchedTransactionsDialogTabBar-content
            $("#"+contentId).css("height", "100%");

            setUpTable(alertsView);
            setUpTable(matchedTransactionsView);
        };

        const getEstimationParam = function (estimationMetrics) {
            const rule = estimationMetrics.ruleDecisionMetrics.rule;

            return {
                estimationId: estimationMetrics.estimation.id,
                ruleId: (rule.isAllRules ? null : rule.id),
                allRuleIds: (rule.isAllRules ? rule.allRuleIds : null),
                forAlerts: estimationMetrics.forAlerts,
            };
        };

        const exportToExcelButton = new sas.hc.m.Button({
            icon: Icon.EXCEL,
            tooltip: "Export to Excel", //sas.fscf.rb.getText("fcm.overview.delete.txt"),
            visible: false,
        } );

        const setExportToExcelButtonVisiblity = function (tabId) {
            exportToExcelButton.setVisible((tabId !== alertsTabId));
        };

        const responsiveToolbar = new sas.hc.m.ResponsiveToolbar({
            width: "10%",
            content:[
                new sas.hc.m.ToolbarSpacer({
                } ),
                exportToExcelButton,
            ],
        } );

        const tabBar = new sas.hc.m.IconTabBar({
            id:  myId + "TabBar",
            items: [],
        } );
        tabBar.addResponsiveToolBar(responsiveToolbar);
        tabBar.addEventDelegate({
            onAfterRendering: function (event) {
                setExportToExcelButtonVisiblity(event.srcControl.getSelectedItem().sId);
            },
        } );
        tabBar.attachTabChanging(function (event) {
            const selectedTab = event.getParameter("newTab");

            setExportToExcelButtonVisiblity(selectedTab.sId);
        } );

        exportToExcelButton.attachPress(function () {
            controller.exportTransactionsToExcel();
        } );

        const renderAlerts = function () {
            if (estimationRuleMetrics.forAlerts) {
                const param = getEstimationParam(estimationRuleMetrics);
                alertsView.setEstimation(param);
                alertsView.lazyLoadContent();
                alertsView.addEventDelegate({
                    onAfterRendering: function () {
                        resizeTabContainerContent();
                    },
                } );
                tabBar.addItem(alertsTab);

                const condition =  function () {
                    const table = alertsView.getTable();
                    return table && table.getModel().getData().items;
                };
                const callback = function () {
                    setTabTooltip(alertsView);
                };
                waitThenDo(condition, callback);

                // Need to hide the transaction view.  Otherwise, when resizeTabContainerContent() is called the table of alertsView is
                // expanded to full height of its parent (as expected).  However, it then shrunk and then expanded again!
                matchedTransactionsView.setVisible(false);
                matchedTransactionsTab.attachBrowserEvent("click", function () {
                    matchedTransactionsView.setVisible(true);
                } );
            }
        };

        const renderMatchedTransactions = function () {
            const param = getEstimationParam(estimationRuleMetrics);

            matchedTransactionsView.setEstimation(param);
            matchedTransactionsView.lazyLoadContent();
            matchedTransactionsView.addEventDelegate({
                onAfterRendering: function () {
                    resizeTabContainerContent();
                },
            } );
            if (estimationRuleMetrics.forAlerts) {
                matchedTransactionsTab.setText(getText("rules.estimation.transactions.alerted.label"));
            }
            tabBar.addItem(matchedTransactionsTab);

            const condition =  function () {
                const table = matchedTransactionsView.getTable();
                return table && table.getModel().getData().items;
            };
            const callback = function () {
                setTabTooltip(matchedTransactionsView);
            };
            waitThenDo(condition, callback);
        };

        const matchedTransactionsDialog = new sas.hc.m.Dialog({
        	id: dialogId,
			title: {
                path: "/data/ruleDecisionMetrics/rule",
				formatter: function (rule) {
                    const answer = rule.isAllRules
                            ? getText("rules.estimation.rule.matched.transactions.name.fmt.title", [getText("rules.estimation.all.rules.label")])
                            : getText("rules.estimation.rule.matched.transactions.fmt.title", [rule.name , rule.displayIdString]);
					return answer;
				},
            },
    	    content: [tabBar],
			contentHeight: "95%",
            contentWidth: "95%",
            cssMaxWidth: ONE_HUNDRED_PERCENT,
            cssMaxHeight: ONE_HUNDRED_PERCENT,
    	    endButton: EstimationHelper.newCloseButton(),
            beforeOpen: [controller.handleMatchedTransactionsDialogBeforeOpen, controller],
            afterClose: function () {
                if (alertsTab) {
                    alertsTab.destroy();
                }
                controller.handleMatchedTransactionsDialogAfterClose();
            },
        } ).addEventDelegate({
            onBeforeRendering: function (event) {
                if (estimationRuleMetrics === null) {
                    estimationRuleMetrics = event.srcControl.getModel().getData().data;
                }

                renderAlerts();
                renderMatchedTransactions();
            },
            onAfterRendering: function (event) {
                reduceSpaceAboveDialogButtons(event.srcControl);
            },
        } );

        return matchedTransactionsDialog;
    },
} );
