sap.ui.jsfragment("sas.fscf.views.fragments.ShowHistoryDialog", {

	_selectedRule: null,
	_createVersionButton: null,
	_dialog: null,
	_isRuleStateAllowsCreateVersion: false, // RAPTOR-1783 Do not show Create Version Button when this is false.
	_model: new sap.ui.model.json.JSONModel({}),
	_printButton: null,                     // RAPTOR-2525
	_ruleVersionTable: null,

	getSelectedRule: function() {
		return this._selectedRule;
	},

	setSelectedRule: function(rule) {
		this._selectedRule = rule;
	},

	isPrintEnabled: function() { // RAPTOR-2525
		return true;  // This will be enabled when the Print feature is implemented with a future story.
	},

	isRuleStateAllowsCreateVersion: function() { // RAPTOR-1783
		return this._isRuleStateAllowsCreateVersion;
	},

	setIsRuleStateAllowsCreateVersion: function(isAllowed) {
		this._isRuleStateAllowsCreateVersion = isAllowed;
	},

    _columns: [{
		   "property":"revision",
		   "label":sas.fscf.rb.getText("rules.rulesPage.rules.table.column.version.txt"),
		   "tooltip":"ruleId", // RAPTOR-1783
		   "width":60,
		   "formatter":0,
	   },
	   {
		   "property":"ruleName",
		   "label":sas.fscf.rb.getText("rules.rulesPage.rules.table.column.name.txt"),
		   "width":180,
		   "formatter":0
	   },
	   {
		   "property":"actionLabel",
		   "label":sas.fscf.rb.getText("rules.rulesPage.rules.table.column.action.txt"),
		   "width":200,
		   "formatter":0
	   },
	   {
		   "property":"comment",
		   "label":sas.fscf.rb.getText("rules.rulesPage.rules.table.column.notes.txt"),
		   "width":300,
		   "formatter":0
	   },
	   {
		   "property":"createUser",
		   "label":sas.fscf.rb.getText("rules.rulesPage.rules.table.column.lastUpdatedBy.txt"),
		   "width":150,
		   "formatter":0
	   },
	   {
		   "property":"createTimestamp",
		   "label":sas.fscf.rb.getText("rules.rulesPage.rules.table.column.lastUpdatedTime.txt"),
		   "width":200,
		   "formatter":"DateTime"
	   }
	],

	createContent: function(oController) {
	},

	lazyLoadContent: function() {
		jQuery.sap.require("sas.hc.m.Dialog");
		jQuery.sap.require("sas.hc.m.MessageDialog");
		jQuery.sap.require("sas.hc.ui.table.Table");
		jQuery.sap.require("sas.hc.ui.table.Column");
		jQuery.sap.require("sas.hc.m.Button");
        jQuery.sap.require("sas.hc.m.Label");

		const _this = this;
		const oTable = new sas.hc.ui.table.Table({
			id: this.sId + 'Table',
			editable: false,
			showTableOptions: false,
			visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Fixed,
			visibleRowCount: 15,
            showAlternateRowShading: true,
			selectionMode: sas.hc.ui.table.SelectionMode.Single,  // RAPTOR-1783
			selectionBehavior: sas.hc.ui.table.SelectionBehavior.RowOnly,
			rememberSelections: true,
            rowSelectionChange: function () {
            	handleVersionSelectionChanged();
            },
		} );
		this._ruleVersionTable = oTable;

		this._columns.forEach(function (column) {
			oTable.addColumn(new sas.hc.ui.table.Column({
				label: _this.getHeaderLabel(column.label),
				template: new sas.hc.m.Text({
					text: {path: (column.parts) ? undefined : column.property,
						   parts: (column.parts) ? column.parts : undefined,
                           formatter: _this.getTextFormatter(column.formatter)
					},
					tooltip: column.tooltip
						? {
                      		path: column.tooltip,
                      		formatter: function(val) {
                      			return val ? ("" + val) : undefined;
                      		}
                      	}
                        : undefined,
						wrapping: false
				} ),
				sortProperty: column.property,
				visible: true,
				width: column.width + "px"
			} ) );
		} );
		oTable.setModel(this.getModel());
		oTable.bindRows("/items");

		const closeButton = new sas.hc.m.Button({
	    	text: sas.fscf.rb.getText('fcm.application.close.txt'),
	        press: function() {
	        	oDialog.close();
	        }
	    } );

		const handleVersionSelectionChanged = function () {
			const selectedVersion = _this.getSeletedVersion();
			const isAVersionSelected = (selectedVersion !== null);
			const isEnableCreateVersionButton = isAVersionSelected && selectedVersion.isCreateVersionAllowed;

			if (_this.isRuleStateAllowsCreateVersion()) {
				_this.getCreateVersionButton().setEnabled(isEnableCreateVersionButton);
			}
			if (_this.isPrintEnabled()) {
				_this.getPrintButton().setEnabled(isAVersionSelected);
			}
		};

		const RuleSystemCode = {
			APPROVED_PRODUCTION: 3,
//			ARCHIVED: 9,
//			DEACTIVATED: 4,
			TRASHED: 5,
		};

		const isCreateVersionAllowed = function (code) {
			return [RuleSystemCode.APPROVED_PRODUCTION, RuleSystemCode.TRASHED].indexOf(code) !== -1;
		};

		const prepRuleHistoryData = function (ruleHistory) {
			ruleHistory.forEach(function (each) {
				// Add the isCreateVersionAllowed property.
				each.isCreateVersionAllowed = isCreateVersionAllowed(each.action);
			} );
		};

        const oDialog = new sas.hc.m.Dialog({
        	id: this.sId + "HistoryDialog",
			title: sas.fscf.rb.getText('rules.rulesPage.rules.history.ruleHistory.txt'),
			content: [
				this.newSpacer({height: "1rem"}),
				oTable
			],
    	    buttons: [],
    	    contentWidth: (window.innerWidth * 0.7) + "px"
        });
		if (this.isPrintEnabled()) {                  // RAPTOR-2525
			oDialog.addButton(this.getPrintButton()); // insertButton() doesn't work!
		}
		if (this.isRuleStateAllowsCreateVersion()) {
			oDialog.addButton(this.getCreateVersionButton());
		}
		oDialog.addButton(closeButton);
        oDialog.attachAfterClose(function(_oEvent) {
			[_this._createVersionButton, _this._printButton, _this._ruleVersionTable, _this._dialog].forEach(function (each) {
				if (each) {
					each.destroy();
				}
			} );
        	this.destroy();
        });
        this._dialog = oDialog;

        const url = "rest/businessunits/" + _this.getBusinessUnitId() + "/rules/" + _this.getSelectedRule().baseRuleId + "/history";
		sas.ajax({url: url, type: "GET", headers:sas.fscf.getAjaxHeaders()})
			.done(function(oData){
				prepRuleHistoryData(oData.items); // RAPTOR-1783
				_this.getModel().setData(oData);
				oTable.sort(oTable.getColumns()[5], sas.hc.ui.table.SortOrder.Descending);
			} )
			.fail(function(oObj, _textStatus, _errorThrown) {
				const message = (oObj.responseJSON && oObj.responseJSON.message) ? oObj.responseJSON.message : (oObj.status + " " + oObj.statusText);

				sas.fscf.showMessageStrip(sap.ui.core.MessageType.Error, message, oDialog.sId);
			} );

        return oDialog;
    },

	getHeaderLabel: function(label) {
		return new sas.hc.m.Label({
			text: label
		});
	},

	getTextFormatter: function(type) {
		if (type == "DateTime") {
			return function(val) {
				if (val == undefined || val == null) {
					return val;
				}
				var date = new Date(val);
				if (isNaN(date.getTime())) {
					return val;
				}
				return sas.fscf.dateTimeFormatter.format(date, true);	// true means use UTC
			}
		} else {
			return function(val) {
				return val;
			}
		}
	},

	handleCreateNewVersionPress: function(_e) {
		const selectedVersion = this.getSeletedVersion();
		if (selectedVersion) {
			this.createNewVersion(selectedVersion);
		}
	},

	createNewVersion: function(selectedVersion) {
		const dialog = this.getDialog();

		this.getRulesPageView().createNewVersion(this.getSelectedRule(),
			selectedVersion.ruleId,
			dialog.getId(),
			function () {dialog.close();}
		);
	},

	// BEGIN: RAPTOR-1783
	getBusinessUnitId: function () {
		return sas.fscf.getCurrentPageView().getBUId();
	},

    getCreateVersionButton: function () {
    	if (this._createVersionButton === null) {
    		this._createVersionButton = this.newCreateVersionButton();
    	}
    	return this._createVersionButton;
    },

	getDialog: function () {
		return this._dialog;
	},

	getModel: function () {
		return this._model;
	},

	getRulesPageView: function () {
		return sap.ui.getCore().byId("RulesPageRulesView");
	},

	getRuleVersionTable: function () {
		return this._ruleVersionTable;
	},

	getSeletedVersion: function () {
		const versionTable = this.getRuleVersionTable();
		const selectedVersions = versionTable.getSelectedIndices().map(function (each) {
			const context = versionTable.getContextByIndex(each);
			const path = context.sPath;

			return this.getModel().getProperty(path);
		}, this );

		return (selectedVersions.length > 0) ? selectedVersions[0] : null;
	},

	newCreateVersionButton: function() { // RAPTOR-1783
    	return new sas.hc.m.Button({
			id: this.sId + "CreateNewVersionButton",
			text: sas.fscf.rb.getText("rules.rulesPage.rules.actions.createNewVersion.txt"),
			tooltip: sas.fscf.rb.getText("rules.rulesPage.rules.actions.createNewVersion.txt"),
			enabled: false,
			press: [this.handleCreateNewVersionPress, this],
		} );
    },

	newSpacer: function (dimension) {
		const Default = {
			HEIGHT: "0rem",
			WIDTH: "1rem",
		};
		const height = dimension && dimension.height ? dimension.height : Default.HEIGHT;
		const width = dimension && dimension.width ? dimension.width : Default.WIDTH;

		return new sas.hc.ui.commons.layout.AbsoluteLayout({
			height: height,
			width: width,
		});
	},
	// END: RAPTOR-1783

	// BEGIN: RAPTOR-2525
	getPrintButton: function() {
		if (this._printButton === null) {
			this._printButton = this.newPrintButton();
		}
		return this._printButton;
	},

	getRuleUrl: function (ruleId) {
		return "rest/businessunits/" + this.getBusinessUnitId() + "/rules/" + ruleId + "?fields=id,baseRuleId,revision,name,description,state,keywords,ruleTypeName,operationalTimeLimit,businessUnitId,alertTypeId,executionOrderType,executionOrderPriority,multiOrgNodeId,multiOrgNodeDisplayName,multiOrgNodeKey,guidedRule,lastUpdateUser,lastUpdateTimestamp,transactionTypeId,queueId,alertReason,pendingApproval,ruleCode,operationalState,configuredOperationalState,metrics";
	},

	handlePrintPress: function () {
		const ruleVersion = this.getSeletedVersion();

		if (ruleVersion) {
			sas.fscf.removeAllMessageStrips();
			this.printVersion(ruleVersion);
		}
	},

	newPrintButton: function() {
		return new sas.hc.m.Button({
			id: this.sId + "PrintButton",
			text: sas.fscf.rb.getText("fcm.application.print.txt"),
			tooltip: sas.fscf.rb.getText("fcm.application.print.txt"),
			enabled: false,
			press: [this.handlePrintPress, this],
		} );
	},

	printVersion: function (ruleVersion) {
		const self = this;
		const url = this.getRuleUrl(ruleVersion.ruleId);
		const dialog = this.getDialog();

		sas.ajax({url: url, type: "GET", headers:sas.fscf.getAjaxHeaders()})
			.done(function (rule) {
				self.getRulesPageView().printRules([rule], rule.state);
			} )
			.fail(function(oObj, _textStatus, _errorThrown) {
				const message = (oObj.responseJSON && oObj.responseJSON.message) ? oObj.responseJSON.message : (oObj.status + " " + oObj.statusText);

				sas.fscf.showMessageStrip(sap.ui.core.MessageType.Error, message, dialog.sId);
			} );
	},
	// END: RAPTOR-2525
});
